"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const web3_js_1 = require("@solana/web3.js");
const common_1 = require("@project-serum/common");
const spl_token_1 = require("@solana/spl-token");
const chai_1 = require("chai");
const chai_as_promised_1 = __importDefault(require("chai-as-promised"));
const src_1 = require("../src");
const amman_1 = require("@metaplex-foundation/amman");
const scenarios_1 = require("./utils/scenarios");
(0, chai_1.use)(chai_as_promised_1.default);
describe('fanout', async () => {
    const connection = new web3_js_1.Connection(amman_1.LOCALHOST, 'confirmed');
    const lamportsNeeded = 10000000000;
    let authorityWallet;
    let fanoutSdk;
    beforeEach(async () => {
        authorityWallet = web3_js_1.Keypair.generate();
        let signature = await connection.requestAirdrop(authorityWallet.publicKey, lamportsNeeded);
        await connection.confirmTransaction(signature);
        fanoutSdk = new src_1.FanoutClient(connection, new common_1.NodeWallet(new web3_js_1.Account(authorityWallet.secretKey)));
        signature = await connection.requestAirdrop(authorityWallet.publicKey, lamportsNeeded);
        await connection.confirmTransaction(signature);
    });
    describe('Token membership model', () => {
        it('Creates fanout w/ token, 2 members stake, has 5 random revenue events, and distributes', async () => {
            var _a, _b, _c;
            const membershipMint = await spl_token_1.Token.createMint(connection, authorityWallet, authorityWallet.publicKey, null, 6, spl_token_1.TOKEN_PROGRAM_ID);
            const distBot = new web3_js_1.Keypair();
            await connection.requestAirdrop(distBot.publicKey, lamportsNeeded);
            const supply = 1000000 * 10 ** 6;
            const tokenAcct = await membershipMint.createAccount(authorityWallet.publicKey);
            const { fanout } = await fanoutSdk.initializeFanout({
                totalShares: 0,
                name: `Test${Date.now()}`,
                membershipModel: src_1.MembershipModel.Token,
                mint: membershipMint.publicKey,
            });
            const mint = await spl_token_1.Token.createMint(connection, authorityWallet, authorityWallet.publicKey, null, 6, spl_token_1.TOKEN_PROGRAM_ID);
            const mintAcctAuthority = await mint.createAssociatedTokenAccount(authorityWallet.publicKey);
            const { fanoutForMint, tokenAccount } = await fanoutSdk.initializeFanoutForMint({
                fanout,
                mint: mint.publicKey,
            });
            const fanoutMintAccount = await fanoutSdk.fetch(fanoutForMint, src_1.FanoutMint);
            (0, chai_1.expect)(fanoutMintAccount.mint.toBase58()).to.equal(mint.publicKey.toBase58());
            (0, chai_1.expect)(fanoutMintAccount.fanout.toBase58()).to.equal(fanout.toBase58());
            (0, chai_1.expect)(fanoutMintAccount.tokenAccount.toBase58()).to.equal(tokenAccount.toBase58());
            (0, chai_1.expect)(fanoutMintAccount.totalInflow.toString()).to.equal('0');
            (0, chai_1.expect)(fanoutMintAccount.lastSnapshotAmount.toString()).to.equal('0');
            let totalStaked = 0;
            const members = [];
            await membershipMint.mintTo(tokenAcct, authorityWallet.publicKey, [], supply);
            for (let index = 0; index <= 4; index++) {
                const member = new web3_js_1.Keypair();
                const pseudoRng = Math.floor(supply * Math.random() * 0.138);
                await connection.requestAirdrop(member.publicKey, lamportsNeeded);
                const tokenAcctMember = await membershipMint.createAssociatedTokenAccount(member.publicKey);
                const mintAcctMember = await mint.createAssociatedTokenAccount(member.publicKey);
                await membershipMint.transfer(tokenAcct, tokenAcctMember, authorityWallet.publicKey, [], pseudoRng);
                totalStaked += pseudoRng;
                const ixs = await fanoutSdk.stakeTokenMemberInstructions({
                    shares: pseudoRng,
                    fanout: fanout,
                    membershipMintTokenAccount: tokenAcctMember,
                    membershipMint: membershipMint.publicKey,
                    member: member.publicKey,
                    payer: member.publicKey,
                });
                const tx = await fanoutSdk.sendInstructions(ixs.instructions, [member], member.publicKey);
                if (!!tx.RpcResponseAndContext.value.err) {
                    const txdetails = await connection.getConfirmedTransaction(tx.TransactionSignature);
                    console.log(txdetails, tx.RpcResponseAndContext.value.err);
                }
                const voucher = await fanoutSdk.fetch(ixs.output.membershipVoucher, src_1.FanoutMembershipVoucher);
                (0, chai_1.expect)((_a = voucher.shares) === null || _a === void 0 ? void 0 : _a.toString()).to.equal(`${pseudoRng}`);
                (0, chai_1.expect)((_b = voucher.membershipKey) === null || _b === void 0 ? void 0 : _b.toBase58()).to.equal(member.publicKey.toBase58());
                (0, chai_1.expect)((_c = voucher.fanout) === null || _c === void 0 ? void 0 : _c.toBase58()).to.equal(fanout.toBase58());
                const stake = await membershipMint.getAccountInfo(ixs.output.stakeAccount);
                (0, chai_1.expect)(stake.amount.toString()).to.equal(`${pseudoRng}`);
                members.push({
                    member,
                    membershipTokenAccount: tokenAcctMember,
                    fanoutMintTokenAccount: mintAcctMember,
                    shares: pseudoRng,
                });
            }
            let runningTotal = 0;
            for (let index = 0; index <= 4; index++) {
                const sent = Math.floor(Math.random() * 100 * 10 ** 6);
                await mint.mintTo(mintAcctAuthority, authorityWallet.publicKey, [], sent);
                await mint.transfer(mintAcctAuthority, tokenAccount, authorityWallet.publicKey, [], sent);
                runningTotal += sent;
                const member = members[index];
                const ix = await fanoutSdk.distributeTokenMemberInstructions({
                    distributeForMint: true,
                    fanoutMint: mint.publicKey,
                    membershipMint: membershipMint.publicKey,
                    fanout: fanout,
                    member: member.member.publicKey,
                    payer: distBot.publicKey,
                });
                const tx = await fanoutSdk.sendInstructions(ix.instructions, [distBot], distBot.publicKey);
                if (!!tx.RpcResponseAndContext.value.err) {
                    const txdetails = await connection.getConfirmedTransaction(tx.TransactionSignature);
                    console.log(txdetails, tx.RpcResponseAndContext.value.err);
                }
                const tokenAcctInfo = await connection.getTokenAccountBalance(member.fanoutMintTokenAccount, 'confirmed');
                const diff = ((supply - totalStaked) * sent) / totalStaked;
                const amountDist = (member.shares * diff) / supply;
                (0, chai_1.expect)(tokenAcctInfo.value.amount, `${amountDist}`);
            }
        });
        it('Init', async () => {
            var _a, _b;
            const membershipMint = await spl_token_1.Token.createMint(connection, authorityWallet, authorityWallet.publicKey, null, 6, spl_token_1.TOKEN_PROGRAM_ID);
            const supply = 1000000 * 10 ** 6;
            const tokenAcct = await membershipMint.createAccount(authorityWallet.publicKey);
            await membershipMint.mintTo(tokenAcct, authorityWallet.publicKey, [], supply);
            const { fanout } = await fanoutSdk.initializeFanout({
                totalShares: 0,
                name: `Test${Date.now()}`,
                membershipModel: src_1.MembershipModel.Token,
                mint: membershipMint.publicKey,
            });
            const fanoutAccount = await fanoutSdk.fetch(fanout, src_1.Fanout);
            (0, chai_1.expect)(fanoutAccount.membershipModel).to.equal(src_1.MembershipModel.Token);
            (0, chai_1.expect)(fanoutAccount.lastSnapshotAmount.toString()).to.equal('0');
            (0, chai_1.expect)(fanoutAccount.totalMembers.toString()).to.equal('0');
            (0, chai_1.expect)(fanoutAccount.totalInflow.toString()).to.equal('0');
            (0, chai_1.expect)(fanoutAccount.totalAvailableShares.toString()).to.equal('0');
            (0, chai_1.expect)(fanoutAccount.totalShares.toString()).to.equal(supply.toString());
            (0, chai_1.expect)((_a = fanoutAccount.membershipMint) === null || _a === void 0 ? void 0 : _a.toBase58()).to.equal(membershipMint.publicKey.toBase58());
            (0, chai_1.expect)((_b = fanoutAccount.totalStakedShares) === null || _b === void 0 ? void 0 : _b.toString()).to.equal('0');
        });
        it('Init For mint', async () => {
            const membershipMint = await spl_token_1.Token.createMint(connection, authorityWallet, authorityWallet.publicKey, null, 6, spl_token_1.TOKEN_PROGRAM_ID);
            const supply = 1000000 * 10 ** 6;
            const tokenAcct = await membershipMint.createAccount(authorityWallet.publicKey);
            await membershipMint.mintTo(tokenAcct, authorityWallet.publicKey, [], supply);
            const { fanout } = await fanoutSdk.initializeFanout({
                totalShares: 0,
                name: `Test${Date.now()}`,
                membershipModel: src_1.MembershipModel.Token,
                mint: membershipMint.publicKey,
            });
            const mint = await spl_token_1.Token.createMint(connection, authorityWallet, authorityWallet.publicKey, null, 6, spl_token_1.TOKEN_PROGRAM_ID);
            const { fanoutForMint, tokenAccount } = await fanoutSdk.initializeFanoutForMint({
                fanout,
                mint: mint.publicKey,
            });
            const fanoutMintAccount = await fanoutSdk.fetch(fanoutForMint, src_1.FanoutMint);
            (0, chai_1.expect)(fanoutMintAccount.mint.toBase58()).to.equal(mint.publicKey.toBase58());
            (0, chai_1.expect)(fanoutMintAccount.fanout.toBase58()).to.equal(fanout.toBase58());
            (0, chai_1.expect)(fanoutMintAccount.tokenAccount.toBase58()).to.equal(tokenAccount.toBase58());
            (0, chai_1.expect)(fanoutMintAccount.totalInflow.toString()).to.equal('0');
            (0, chai_1.expect)(fanoutMintAccount.lastSnapshotAmount.toString()).to.equal('0');
        });
        it('Stakes Members', async () => {
            var _a, _b, _c, _d, _e;
            const membershipMint = await spl_token_1.Token.createMint(connection, authorityWallet, authorityWallet.publicKey, null, 6, spl_token_1.TOKEN_PROGRAM_ID);
            const supply = 1000000 * 10 ** 6;
            const member = new web3_js_1.Keypair();
            await connection.requestAirdrop(member.publicKey, lamportsNeeded);
            const tokenAcct = await membershipMint.createAccount(authorityWallet.publicKey);
            const tokenAcctMember = await membershipMint.createAssociatedTokenAccount(member.publicKey);
            await membershipMint.mintTo(tokenAcct, authorityWallet.publicKey, [], supply);
            await membershipMint.transfer(tokenAcct, tokenAcctMember, authorityWallet.publicKey, [], supply * 0.1);
            const { fanout } = await fanoutSdk.initializeFanout({
                totalShares: 0,
                name: `Test${Date.now()}`,
                membershipModel: src_1.MembershipModel.Token,
                mint: membershipMint.publicKey,
            });
            const ixs = await fanoutSdk.stakeTokenMemberInstructions({
                shares: supply * 0.1,
                fanout: fanout,
                membershipMintTokenAccount: tokenAcctMember,
                membershipMint: membershipMint.publicKey,
                member: member.publicKey,
                payer: member.publicKey,
            });
            const tx = await fanoutSdk.sendInstructions(ixs.instructions, [member], member.publicKey);
            if (!!tx.RpcResponseAndContext.value.err) {
                const txdetails = await connection.getConfirmedTransaction(tx.TransactionSignature);
                console.log(txdetails, tx.RpcResponseAndContext.value.err);
            }
            const voucher = await fanoutSdk.fetch(ixs.output.membershipVoucher, src_1.FanoutMembershipVoucher);
            (0, chai_1.expect)((_a = voucher.shares) === null || _a === void 0 ? void 0 : _a.toString()).to.equal(`${supply * 0.1}`);
            (0, chai_1.expect)((_b = voucher.membershipKey) === null || _b === void 0 ? void 0 : _b.toBase58()).to.equal(member.publicKey.toBase58());
            (0, chai_1.expect)((_c = voucher.fanout) === null || _c === void 0 ? void 0 : _c.toBase58()).to.equal(fanout.toBase58());
            const stake = await membershipMint.getAccountInfo(ixs.output.stakeAccount);
            (0, chai_1.expect)(stake.amount.toString()).to.equal(`${supply * 0.1}`);
            const fanoutAccountData = await fanoutSdk.fetch(fanout, src_1.Fanout);
            (0, chai_1.expect)((_d = fanoutAccountData.totalShares) === null || _d === void 0 ? void 0 : _d.toString()).to.equal(`${supply}`);
            (0, chai_1.expect)((_e = fanoutAccountData.totalStakedShares) === null || _e === void 0 ? void 0 : _e.toString()).to.equal(`${supply * 0.1}`);
        });
        it('Allows Authority to Stake Members', async () => {
            var _a, _b, _c, _d, _e;
            const membershipMint = await spl_token_1.Token.createMint(connection, authorityWallet, authorityWallet.publicKey, null, 6, spl_token_1.TOKEN_PROGRAM_ID);
            const supply = 1000000 * 10 ** 6;
            const member = new web3_js_1.Keypair();
            await connection.requestAirdrop(member.publicKey, lamportsNeeded);
            const tokenAcct = await membershipMint.createAccount(authorityWallet.publicKey);
            await membershipMint.mintTo(tokenAcct, authorityWallet.publicKey, [], supply);
            const { fanout } = await fanoutSdk.initializeFanout({
                totalShares: 0,
                name: `Test${Date.now()}`,
                membershipModel: src_1.MembershipModel.Token,
                mint: membershipMint.publicKey,
            });
            const ixs = await fanoutSdk.stakeForTokenMemberInstructions({
                shares: supply * 0.1,
                fanout: fanout,
                membershipMintTokenAccount: tokenAcct,
                membershipMint: membershipMint.publicKey,
                fanoutAuthority: authorityWallet.publicKey,
                member: member.publicKey,
                payer: authorityWallet.publicKey,
            });
            const tx = await fanoutSdk.sendInstructions(ixs.instructions, [], authorityWallet.publicKey);
            if (!!tx.RpcResponseAndContext.value.err) {
                const txdetails = await connection.getConfirmedTransaction(tx.TransactionSignature);
                console.log(txdetails, tx.RpcResponseAndContext.value.err);
            }
            const voucher = await fanoutSdk.fetch(ixs.output.membershipVoucher, src_1.FanoutMembershipVoucher);
            (0, chai_1.expect)((_a = voucher.shares) === null || _a === void 0 ? void 0 : _a.toString()).to.equal(`${supply * 0.1}`);
            (0, chai_1.expect)((_b = voucher.membershipKey) === null || _b === void 0 ? void 0 : _b.toBase58()).to.equal(member.publicKey.toBase58());
            (0, chai_1.expect)((_c = voucher.fanout) === null || _c === void 0 ? void 0 : _c.toBase58()).to.equal(fanout.toBase58());
            const stake = await membershipMint.getAccountInfo(ixs.output.stakeAccount);
            (0, chai_1.expect)(stake.amount.toString()).to.equal(`${supply * 0.1}`);
            const fanoutAccountData = await fanoutSdk.fetch(fanout, src_1.Fanout);
            (0, chai_1.expect)((_d = fanoutAccountData.totalShares) === null || _d === void 0 ? void 0 : _d.toString()).to.equal(`${supply}`);
            (0, chai_1.expect)((_e = fanoutAccountData.totalStakedShares) === null || _e === void 0 ? void 0 : _e.toString()).to.equal(`${supply * 0.1}`);
        });
        it('Distribute a Native Fanout with Token Members', async () => {
            var _a, _b;
            const membershipMint = await spl_token_1.Token.createMint(connection, authorityWallet, authorityWallet.publicKey, null, 6, spl_token_1.TOKEN_PROGRAM_ID);
            const distBot = new web3_js_1.Keypair();
            await connection.requestAirdrop(distBot.publicKey, lamportsNeeded);
            const builtFanout = await (0, scenarios_1.builtTokenFanout)(membershipMint, authorityWallet, fanoutSdk, 100, 5);
            (0, chai_1.expect)(builtFanout.fanoutAccountData.totalAvailableShares.toString()).to.equal('0');
            (0, chai_1.expect)(builtFanout.fanoutAccountData.totalMembers.toString()).to.equal('5');
            (0, chai_1.expect)((_a = builtFanout.fanoutAccountData.totalShares) === null || _a === void 0 ? void 0 : _a.toString()).to.equal(`${100 ** 6}`);
            (0, chai_1.expect)((_b = builtFanout.fanoutAccountData.totalStakedShares) === null || _b === void 0 ? void 0 : _b.toString()).to.equal(`${100 ** 6}`);
            (0, chai_1.expect)(builtFanout.fanoutAccountData.lastSnapshotAmount.toString()).to.equal('0');
            await connection.requestAirdrop(builtFanout.fanoutAccountData.accountKey, lamportsNeeded);
            const firstSnapshot = lamportsNeeded;
            const firstMemberAmount = firstSnapshot * 0.2;
            const member1 = builtFanout.members[0];
            const ix = await fanoutSdk.distributeTokenMemberInstructions({
                distributeForMint: false,
                membershipMint: membershipMint.publicKey,
                fanout: builtFanout.fanout,
                member: member1.wallet.publicKey,
                payer: distBot.publicKey,
            });
            const memberBefore = await fanoutSdk.connection.getAccountInfo(member1.wallet.publicKey);
            const tx = await fanoutSdk.sendInstructions(ix.instructions, [distBot], distBot.publicKey);
            if (!!tx.RpcResponseAndContext.value.err) {
                const txdetails = await connection.getConfirmedTransaction(tx.TransactionSignature);
                console.log(txdetails, tx.RpcResponseAndContext.value.err);
            }
            const voucher = await fanoutSdk.fetch(ix.output.membershipVoucher, src_1.FanoutMembershipVoucher);
            const memberAfter = await fanoutSdk.connection.getAccountInfo(member1.wallet.publicKey);
            (0, chai_1.expect)(voucher.lastInflow.toString()).to.equal(`${firstSnapshot}`);
            (0, chai_1.expect)(voucher.shares.toString()).to.equal(`${100 ** 6 / 5}`);
            (0, chai_1.expect)((memberAfter === null || memberAfter === void 0 ? void 0 : memberAfter.lamports) - (memberBefore === null || memberBefore === void 0 ? void 0 : memberBefore.lamports)).to.equal(firstMemberAmount);
        });
        it('Unstake a Native Fanout with Token Members', async () => {
            var _a;
            const membershipMint = await spl_token_1.Token.createMint(connection, authorityWallet, authorityWallet.publicKey, null, 6, spl_token_1.TOKEN_PROGRAM_ID);
            const distBot = new web3_js_1.Keypair();
            const signature = await connection.requestAirdrop(distBot.publicKey, 1);
            await connection.confirmTransaction(signature);
            const builtFanout = await (0, scenarios_1.builtTokenFanout)(membershipMint, authorityWallet, fanoutSdk, 100, 5);
            const sent = 10;
            const beforeUnstake = await fanoutSdk.fetch(builtFanout.fanout, src_1.Fanout);
            await connection.requestAirdrop(builtFanout.fanoutAccountData.accountKey, sent);
            const firstSnapshot = sent * web3_js_1.LAMPORTS_PER_SOL;
            const firstMemberAmount = firstSnapshot * 0.2;
            const member1 = builtFanout.members[0];
            const memberFanoutSdk = new src_1.FanoutClient(connection, new common_1.NodeWallet(new web3_js_1.Account(member1.wallet.secretKey)));
            const ix = await memberFanoutSdk.distributeTokenMemberInstructions({
                distributeForMint: false,
                membershipMint: membershipMint.publicKey,
                fanout: builtFanout.fanout,
                member: member1.wallet.publicKey,
                payer: member1.wallet.publicKey,
            });
            const voucherBefore = await memberFanoutSdk.fetch(ix.output.membershipVoucher, src_1.FanoutMembershipVoucher);
            await memberFanoutSdk.unstakeTokenMember({
                fanout: builtFanout.fanout,
                member: member1.wallet.publicKey,
                payer: member1.wallet.publicKey,
            });
            const afterUnstake = await memberFanoutSdk.fetch(builtFanout.fanout, src_1.Fanout);
            const memberAfter = await memberFanoutSdk.connection.getAccountInfo(member1.wallet.publicKey);
            (0, chai_1.expect)((_a = afterUnstake.totalStakedShares) === null || _a === void 0 ? void 0 : _a.toString()).to.equal(`${(beforeUnstake === null || beforeUnstake === void 0 ? void 0 : beforeUnstake.totalStakedShares).sub(voucherBefore.shares)}`);
        });
    });
});
//# sourceMappingURL=token.js.map