"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const web3_js_1 = require("@solana/web3.js");
const common_1 = require("@project-serum/common");
const spl_token_1 = require("@solana/spl-token");
const chai_1 = require("chai");
const chai_as_promised_1 = __importDefault(require("chai-as-promised"));
const src_1 = require("../src");
const amman_1 = require("@metaplex-foundation/amman");
const scenarios_1 = require("./utils/scenarios");
(0, chai_1.use)(chai_as_promised_1.default);
describe('fanout', async () => {
    const connection = new web3_js_1.Connection(amman_1.LOCALHOST, 'confirmed');
    const lamportsNeeded = 10000000000;
    let authorityWallet;
    let fanoutSdk;
    beforeEach(async () => {
        authorityWallet = web3_js_1.Keypair.generate();
        await connection.requestAirdrop(authorityWallet.publicKey, web3_js_1.LAMPORTS_PER_SOL * 10);
        fanoutSdk = new src_1.FanoutClient(connection, new common_1.NodeWallet(new web3_js_1.Account(authorityWallet.secretKey)));
        await connection.requestAirdrop(authorityWallet.publicKey, web3_js_1.LAMPORTS_PER_SOL * 10);
    });
    describe('Wallet membership model', () => {
        it('Init', async () => {
            const { fanout } = await fanoutSdk.initializeFanout({
                totalShares: 100,
                name: `Test${Date.now()}`,
                membershipModel: src_1.MembershipModel.Wallet,
            });
            const fanoutAccount = await fanoutSdk.fetch(fanout, src_1.Fanout);
            (0, chai_1.expect)(fanoutAccount.membershipModel).to.equal(src_1.MembershipModel.Wallet);
            (0, chai_1.expect)(fanoutAccount.lastSnapshotAmount.toString()).to.equal('0');
            (0, chai_1.expect)(fanoutAccount.totalMembers.toString()).to.equal('0');
            (0, chai_1.expect)(fanoutAccount.totalInflow.toString()).to.equal('0');
            (0, chai_1.expect)(fanoutAccount.totalAvailableShares.toString()).to.equal('100');
            (0, chai_1.expect)(fanoutAccount.totalShares.toString()).to.equal('100');
            (0, chai_1.expect)(fanoutAccount.membershipMint).to.equal(null);
            (0, chai_1.expect)(fanoutAccount.totalStakedShares).to.equal(null);
        });
        it('Init For mint', async () => {
            const { fanout } = await fanoutSdk.initializeFanout({
                totalShares: 100,
                name: `Test${Date.now()}`,
                membershipModel: src_1.MembershipModel.Wallet,
            });
            const mint = await spl_token_1.Token.createMint(connection, authorityWallet, authorityWallet.publicKey, null, 6, spl_token_1.TOKEN_PROGRAM_ID);
            const { fanoutForMint, tokenAccount } = await fanoutSdk.initializeFanoutForMint({
                fanout,
                mint: mint.publicKey,
            });
            const fanoutMintAccount = await fanoutSdk.fetch(fanoutForMint, src_1.FanoutMint);
            (0, chai_1.expect)(fanoutMintAccount.mint.toBase58()).to.equal(mint.publicKey.toBase58());
            (0, chai_1.expect)(fanoutMintAccount.fanout.toBase58()).to.equal(fanout.toBase58());
            (0, chai_1.expect)(fanoutMintAccount.tokenAccount.toBase58()).to.equal(tokenAccount.toBase58());
            (0, chai_1.expect)(fanoutMintAccount.totalInflow.toString()).to.equal('0');
            (0, chai_1.expect)(fanoutMintAccount.lastSnapshotAmount.toString()).to.equal('0');
        });
        it('Init For Wrapped Sol', async () => {
            const { fanout } = await fanoutSdk.initializeFanout({
                totalShares: 100,
                name: `Test${Date.now()}`,
                membershipModel: src_1.MembershipModel.Wallet,
            });
            const { fanoutForMint, tokenAccount } = await fanoutSdk.initializeFanoutForMint({
                fanout,
                mint: spl_token_1.NATIVE_MINT,
            });
            const fanoutMintAccount = await fanoutSdk.fetch(fanoutForMint, src_1.FanoutMint);
            (0, chai_1.expect)(fanoutMintAccount.mint.toBase58()).to.equal(spl_token_1.NATIVE_MINT.toBase58());
            (0, chai_1.expect)(fanoutMintAccount.fanout.toBase58()).to.equal(fanout.toBase58());
            (0, chai_1.expect)(fanoutMintAccount.tokenAccount.toBase58()).to.equal(tokenAccount.toBase58());
            (0, chai_1.expect)(fanoutMintAccount.totalInflow.toString()).to.equal('0');
            (0, chai_1.expect)(fanoutMintAccount.lastSnapshotAmount.toString()).to.equal('0');
        });
        it('Adds Members With Wallet', async () => {
            var _a, _b;
            const init = await fanoutSdk.initializeFanout({
                totalShares: 100,
                name: `Test${Date.now()}`,
                membershipModel: src_1.MembershipModel.Wallet,
            });
            const member = new web3_js_1.Keypair();
            const { membershipAccount } = await fanoutSdk.addMemberWallet({
                fanout: init.fanout,
                fanoutNativeAccount: init.nativeAccount,
                membershipKey: member.publicKey,
                shares: 10,
            });
            const fanoutAccount = await fanoutSdk.fetch(init.fanout, src_1.Fanout);
            const membershipAccountData = await fanoutSdk.fetch(membershipAccount, src_1.FanoutMembershipVoucher);
            (0, chai_1.expect)(fanoutAccount.membershipModel).to.equal(src_1.MembershipModel.Wallet);
            (0, chai_1.expect)(fanoutAccount.lastSnapshotAmount.toString()).to.equal('0');
            (0, chai_1.expect)(fanoutAccount.totalMembers.toString()).to.equal('1');
            (0, chai_1.expect)(fanoutAccount.totalInflow.toString()).to.equal('0');
            (0, chai_1.expect)(fanoutAccount.totalAvailableShares.toString()).to.equal('90');
            (0, chai_1.expect)(fanoutAccount.totalShares.toString()).to.equal('100');
            (0, chai_1.expect)(fanoutAccount.membershipMint).to.equal(null);
            (0, chai_1.expect)(fanoutAccount.totalStakedShares).to.equal(null);
            (0, chai_1.expect)((_a = membershipAccountData === null || membershipAccountData === void 0 ? void 0 : membershipAccountData.shares) === null || _a === void 0 ? void 0 : _a.toString()).to.equal('10');
            (0, chai_1.expect)((_b = membershipAccountData === null || membershipAccountData === void 0 ? void 0 : membershipAccountData.membershipKey) === null || _b === void 0 ? void 0 : _b.toBase58()).to.equal(member.publicKey.toBase58());
        });
        it('Distribute a Native Fanout with Wallet Members', async () => {
            const builtFanout = await (0, scenarios_1.builtWalletFanout)(fanoutSdk, 100, 5);
            (0, chai_1.expect)(builtFanout.fanoutAccountData.totalAvailableShares.toString()).to.equal('0');
            (0, chai_1.expect)(builtFanout.fanoutAccountData.totalMembers.toString()).to.equal('5');
            (0, chai_1.expect)(builtFanout.fanoutAccountData.lastSnapshotAmount.toString()).to.equal('0');
            const distBot = new web3_js_1.Keypair();
            await connection.requestAirdrop(builtFanout.fanoutAccountData.accountKey, lamportsNeeded);
            await connection.requestAirdrop(distBot.publicKey, lamportsNeeded);
            const member1 = builtFanout.members[0];
            const member2 = builtFanout.members[1];
            const distMember1 = await fanoutSdk.distributeWalletMemberInstructions({
                distributeForMint: false,
                member: member1.wallet.publicKey,
                fanout: builtFanout.fanout,
                payer: distBot.publicKey,
            });
            const distMember2 = await fanoutSdk.distributeWalletMemberInstructions({
                distributeForMint: false,
                member: member2.wallet.publicKey,
                fanout: builtFanout.fanout,
                payer: distBot.publicKey,
            });
            const holdingAccountReserved = await connection.getMinimumBalanceForRentExemption(1);
            const memberDataBefore1 = await connection.getAccountInfo(member1.wallet.publicKey);
            const memberDataBefore2 = await connection.getAccountInfo(member2.wallet.publicKey);
            const holdingAccountBefore = await connection.getAccountInfo(builtFanout.fanoutAccountData.accountKey);
            (0, chai_1.expect)(memberDataBefore2).to.be.null;
            (0, chai_1.expect)(memberDataBefore1).to.be.null;
            const firstSnapshot = lamportsNeeded;
            (0, chai_1.expect)((holdingAccountBefore === null || holdingAccountBefore === void 0 ? void 0 : holdingAccountBefore.lamports) + lamportsNeeded).to.equal(firstSnapshot + holdingAccountReserved);
            const tx = await fanoutSdk.sendInstructions([...distMember1.instructions, ...distMember2.instructions], [distBot], distBot.publicKey);
            if (!!tx.RpcResponseAndContext.value.err) {
                const txdetails = await connection.getConfirmedTransaction(tx.TransactionSignature);
                console.log(txdetails, tx.RpcResponseAndContext.value.err);
            }
            const memberDataAfter1 = await connection.getAccountInfo(member1.wallet.publicKey);
            const memberDataAfter2 = await connection.getAccountInfo(member2.wallet.publicKey);
            const holdingAccountAfter = await connection.getAccountInfo(builtFanout.fanoutAccountData.accountKey);
            const membershipAccount1 = await fanoutSdk.fetch(member1.voucher, src_1.FanoutMembershipVoucher);
            (0, chai_1.expect)(memberDataAfter1 === null || memberDataAfter1 === void 0 ? void 0 : memberDataAfter1.lamports).to.equal(firstSnapshot * 0.2);
            (0, chai_1.expect)(memberDataAfter2 === null || memberDataAfter2 === void 0 ? void 0 : memberDataAfter2.lamports).to.equal(firstSnapshot * 0.2);
            (0, chai_1.expect)(holdingAccountAfter === null || holdingAccountAfter === void 0 ? void 0 : holdingAccountAfter.lamports).to.equal(firstSnapshot - firstSnapshot * 0.4 + holdingAccountReserved);
            (0, chai_1.expect)(builtFanout.fanoutAccountData.lastSnapshotAmount.toString()).to.equal('0');
            (0, chai_1.expect)(membershipAccount1.totalInflow.toString()).to.equal(`${firstSnapshot * 0.2}`);
        });
        it('Transfer Shares', async () => {
            const builtFanout = await (0, scenarios_1.builtWalletFanout)(fanoutSdk, 100, 5);
            const sent = 10;
            await connection.requestAirdrop(builtFanout.fanoutAccountData.accountKey, sent);
            await connection.requestAirdrop(fanoutSdk.wallet.publicKey, 1);
            const member0Wallet = builtFanout.members[0].wallet;
            const member1Wallet = builtFanout.members[1].wallet;
            const member0Voucher = builtFanout.members[0].voucher;
            const member1Voucher = builtFanout.members[1].voucher;
            await fanoutSdk.transferShares({
                fromMember: member0Wallet.publicKey,
                toMember: member1Wallet.publicKey,
                fanout: builtFanout.fanout,
                shares: 20,
            });
            const membershipAccount0 = await fanoutSdk.fetch(member0Voucher, src_1.FanoutMembershipVoucher);
            const membershipAccount1 = await fanoutSdk.fetch(member1Voucher, src_1.FanoutMembershipVoucher);
            (0, chai_1.expect)(membershipAccount0.shares.toString()).to.equal('0');
            (0, chai_1.expect)(membershipAccount1.shares.toString()).to.equal('40');
        });
        it('Remove Member', async () => {
            const builtFanout = await (0, scenarios_1.builtWalletFanout)(fanoutSdk, 100, 5);
            const sent = 10;
            const rando = new web3_js_1.Keypair();
            await connection.requestAirdrop(builtFanout.fanoutAccountData.accountKey, sent);
            await connection.requestAirdrop(fanoutSdk.wallet.publicKey, 1);
            const member0Wallet = builtFanout.members[0].wallet;
            const member1Wallet = builtFanout.members[1].wallet;
            const member0Voucher = builtFanout.members[0].voucher;
            await fanoutSdk.transferShares({
                fromMember: member0Wallet.publicKey,
                toMember: member1Wallet.publicKey,
                fanout: builtFanout.fanout,
                shares: 20,
            });
            await fanoutSdk.removeMember({
                destination: rando.publicKey,
                fanout: builtFanout.fanout,
                member: member0Wallet.publicKey,
            });
            const fanout_after = await fanoutSdk.fetch(builtFanout.fanout, src_1.Fanout);
            (0, chai_1.expect)(fanout_after.totalMembers.toString()).to.equal('4');
            (0, chai_1.expect)(fanoutSdk.getAccountInfo(member0Voucher)).to.be.rejectedWith(new Error('Account Not Found'));
        });
    });
});
//# sourceMappingURL=wallet.js.map